﻿//////////////////////////////////////////////
// RenderScenePass.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Camera ;
	class BlendState ;
	class DepthStencilState ;
	class RasterState ;
	class RenderQueue ;
}

/// Includes ---------------------------------

// nkGraphics
#include "Pass.h"

#include "../Dll/DllDefines.h"

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT RenderScenePass : public Pass
	{
		protected :

			// Attributs
			// Les RQ à prendre en compte
			int _minRQ ;
			int _maxRQ ;
			// Ou une RQ spéciale attitrée
			RenderQueue* _specialRQ ;

			// Camera si besoin est
			Camera* _renderCam ;
			Camera* _cullingCam ;
			Camera* _lodCam ;

			// States
			BlendState* _blendState ;
			RasterState* _rasterState ;
			DepthStencilState* _depthState ;

			// Transparent
			bool _transparentPass ;

		protected :

			// Functions
			// Constructor, destructor
			RenderScenePass () ;
			virtual ~RenderScenePass () ;
		
		public :
		
			// Getters
			int getMinRQ () const ;
			int getMaxRQ () const ;
			RenderQueue* getSpecialRQ () const ;
			Camera* getRenderCam () const ;
			Camera* getCullingCam () const ;
			Camera* getLodCam () const ;
			BlendState* getBlendState () const ;
			RasterState* getRasterState () const ;
			DepthStencilState* getDepthStencilState () const ;
			bool isTransparentPass () const ;

			// Setters
			void setMinRQ (int value) ;
			void setMaxRQ (int value) ;
			void setSpecialRQ (const char* name) ;
			void setRenderCam (Camera* cam) ;
			void setCullingCam (Camera* cam) ;
			void setLodCam (Camera* cam) ;
			virtual void setBlendState (BlendState* state) ;
			virtual void setRasterState (RasterState* state) ;
			virtual void setDepthStencilState (DepthStencilState* state) ;
			void setTransparentPass (bool value) ;

			// L'export et import
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}