﻿//////////////////////////////////////////////
// ProgramSourcesHolder.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Standards
#include <string>
#include <vector>

/// Internals --------------------------------

namespace nkGraphics
{
	struct DLL_GRAPHICS_EXPORT MacroEntry
	{
		std::string _key ;
		std::string _value ;
	} ;
}

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ProgramSourcesHolder final
	{
		private :

			// Les mémoires
			// Ce seront des char* pour éviter les différences 
			std::string _vertexMemory ;
			std::string _hullMemory ;
			std::string _domainMemory ;
			std::string _geometryMemory ;
			std::string _pixelMemory ;
			std::string _computeMemory ;
			std::string _raytracingMemory ;

			std::vector<MacroEntry> _macrosVertex ;
			std::vector<MacroEntry> _macrosHull ;
			std::vector<MacroEntry> _macrosDomain ;
			std::vector<MacroEntry> _macrosGeometry ;
			std::vector<MacroEntry> _macrosPixel ;
			std::vector<MacroEntry> _macrosCompute ;
			std::vector<MacroEntry> _macrosRaytracing ;

		public :

			// Constructeur, destructeur
			ProgramSourcesHolder () ;
			~ProgramSourcesHolder () ;

			// Getters
			std::string_view getVertexMemory () const ;
			std::string_view getHullMemory () const ;
			std::string_view getDomainMemory () const ;
			std::string_view getGeometryMemory () const ;
			std::string_view getPixelMemory () const ;
			std::string_view getComputeMemory () const ;
			std::string_view getRaytracingMemory () const ;
			const std::vector<MacroEntry>& getDefineMacrosVertex () const ;
			const std::vector<MacroEntry>& getDefineMacrosHull () const ;
			const std::vector<MacroEntry>& getDefineMacrosDomain () const ;
			const std::vector<MacroEntry>& getDefineMacrosGeometry () const ;
			const std::vector<MacroEntry>& getDefineMacrosPixel () const ;
			const std::vector<MacroEntry>& getDefineMacrosCompute () const ;
			const std::vector<MacroEntry>& getDefineMacrosRaytracing () const ;
			unsigned int getDefineMacroMaxNumber () const ;
			unsigned int getDefineMacroMaxNumberPipeline () const ;
			unsigned int getDefineMacroNumberCompute () const ;
			unsigned int getDefineMacroNumberRaytracing () const ;

			// Setters
			void setVertexMemory (const std::string_view& str) ;
			void setHullMemory (const std::string_view& str) ;
			void setDomainMemory (const std::string_view& str) ;
			void setGeometryMemory (const std::string_view& str) ;
			void setPixelMemory (const std::string_view& str) ;
			void setComputeMemory (const std::string_view& str) ;
			void setRaytracingMemory (const std::string_view& str) ;
			void addDefineMacroVertex (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroHull (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroDomain (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroGeometry (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroPixel (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroCompute (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroRaytracing (const std::string_view& key, const std::string_view& value) ;

			// Utilitaires
			bool hasMinimalDefinitionsPipeline () const ;
			bool hasMinimalDefinitionsCompute () const ;
			bool hasMinimalDefinitionsGeometry () const ;
			bool hasMinimalDefinitionsRaytracing () const ;
	} ;
}