﻿//////////////////////////////////////////////
// RasterStateManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class RasterState ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <unordered_map>
#include <mutex>
#include <string>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT RasterStateManager : public nkCommon::SingletonClass<RasterStateManager>
	{
		friend class nkCommon::SingletonClass<RasterStateManager> ;

		private :

			// Attributs
			// Liste
			std::unordered_map<std::string, RasterState*> _memory ;

			// Default state
			RasterState* _defaultState ;

			// Mutex
			std::mutex _mutex ;

		private :

			// Functions
			// Constructeur
			RasterStateManager () ;
		
		public :

			// Destructeur
			~RasterStateManager () ;
		
			// Manipulation mémoire
			RasterState* createOrRetrieve (const std::string_view& name) ;
			RasterState* get (const std::string_view& name) ;
			RasterState* getByIndex (unsigned int index) ;
			void rename (const std::string_view& oldName, const std::string_view& newName) ;
			void erase (const std::string_view& name) ;

			// Getters
			RasterState* getDefaultState () const ;
	} ;
}