﻿//////////////////////////////////////////////
// Vector.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkMaths
{
	class IVector ;
	class Matrix ;
	class Quaternion ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/ByteAlignedClass.h>

// Windows / Dx
#include <DirectXMath.h>

// Standards
#include <string_view>

/// Class ------------------------------------

namespace nkMaths
{
	class DLL_MATHS_EXPORT Vector : public ByteAlignedClass<16>
	{
		friend class IVector ;

		protected :
		
			// Attributs
			DirectX::XMVECTOR _values ;
		
		public :
		
			// Constructeur, destructeur
			Vector () ;
			Vector (float x, float y) ;
			Vector (float x, float y, float z) ;
			Vector (float x, float y, float z, float w) ;
			Vector (const Vector& other) ;
			Vector (const IVector& other) ;
			Vector (const DirectX::XMVECTOR& value) ;
			~Vector () ;

			// Getters
			float getX () const ;
			float getY () const ;
			float getZ () const ;
			float getW () const ;
			DirectX::XMVECTOR getValues () const ;

			// Setters
			void setX (float value) ;
			void setY (float value) ;
			void setZ (float value) ;
			void setW (float value) ;

			// Ajout à une composante
			void addX (float value) ;
			void addY (float value) ;
			void addZ (float value) ;
			void addW (float value) ;
			void multX (float value) ;
			void multY (float value) ;
			void multZ (float value) ;
			void multW (float value) ;

			// Opérateurs plus avancés
			// Normalisation
			void normalizeAsVec2 () ;
			Vector getNormalizedAsVec2 () const ;
			void normalizeAsVec3 () ;
			Vector getNormalizedAsVec3 () const ;
			void normalizeAsVec4 () ;
			Vector getNormalizedAsVec4 () const ;
			// Dot product
			float dotProductAsVec2 (const Vector& other) const ;
			float dotProductAsVec3 (const Vector& other) const ;
			float dotProductAsVec4 (const Vector& other) const ;
			// Cross product
			void setAsCrossVec2 (const Vector& other) ;
			Vector getCrossVec2 (const Vector& other) const ;
			void setAsCrossVec3 (const Vector& other) ;
			Vector getCrossVec3 (const Vector& other) const ;
			void setAsCrossVec4 (const Vector& other1, const Vector& other2) ;
			Vector getCrossVec4 (const Vector& other1, const Vector& other2) const ;
			// Distances
			float getLength () const ;
			float getLengthSquared () const ;
			float getDistanceSquared (const Vector& other) const ;
			float getDistance (const Vector& other) const ;

			// Pratique
			void fromString (const std::string_view& str) ;

			// Opérateurs de base
			// Egalité
			Vector& operator= (const Vector& other) ;
			Vector& operator= (const IVector& other) ;
			// Addition
			Vector operator+ (const Vector& other) const ;
			void operator+= (const Vector& other) ;
			// Soustraction
			Vector operator- (const Vector& other) const ;
			void operator-= (const Vector& other) ;
			// Multiplication
			Vector operator* (const Vector& other) const ;
			void operator*= (const Vector& other) ;
			Vector operator* (const Quaternion& other) const ;
			void operator*= (const Quaternion& other) ;
			Vector operator* (const Matrix& mat) const ;
			void operator*= (const Matrix& other) ;
			Vector operator* (float coeff) const ;
			void operator*= (float coeff) ;
			// Division
			Vector operator/ (const Vector& other) const ;
			void operator/= (const Vector& other) ;
			Vector operator/ (float coeff) const ;
			void operator/= (float coeff) ;
			// Comparaison
			bool operator== (const Vector& other) const ;
			bool operator!= (const Vector& other) const ;
			bool operator< (const Vector& other) const ;
			bool operator<= (const Vector& other) const ;
			bool operator> (const Vector& other) const ;
			bool operator>= (const Vector& other) const ;
	} ;
}